/*
 * $Id: EchoRegist.cpp 711 2008-02-10 18:52:44Z sayer $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 * Copyright (C) 2006 iptego GmbH
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "EchoRegist.h"
#include "AmConfig.h"
#include "AmUtils.h"

#include "sems.h"
#include "log.h"

#include "ampi/UACAuthAPI.h"
#include "ampi/SIPRegistrarClientAPI.h"
#include "AmUAC.h"
#include "AmPlugIn.h"

#define MOD_NAME "echo_regist"

EXPORT_SESSION_FACTORY(EchoRegistFactory,MOD_NAME);

string EchoRegistFactory::AnnouncePath;
string EchoRegistFactory::AnnounceFile;

EchoRegistFactory::EchoRegistFactory(const string& _app_name)
  : AmSessionFactory(_app_name)
{
}

int EchoRegistFactory::onLoad()
{
    AmConfigReader cfg;
    if(cfg.loadFile(AmConfig::ModConfigPath + string(MOD_NAME ".conf")))
	return -1;

    // get application specific global parameters
    configureModule(cfg);

    // stay backwards compatible
    RegInfo ri; 
    ri.domain = cfg.getParameter("domain","");
    ri.user = cfg.getParameter("user","");
    ri.display_name = cfg.getParameter("display_name","");
    ri.auth_user = cfg.getParameter("auth_user","");
    ri.passwd = cfg.getParameter("pwd","");

    dialer.add_reg(ri);
    dialer.start();

    session_timer_f = AmPlugIn::instance()->getFactory4Seh("session_timer");
    if(session_timer_f == NULL)
      ERROR("unable to get a session_timer\n");

    return 0;
}

AmSession* EchoRegistFactory::onInvite(const AmSipRequest& req)
{
    
   EchoRegistDialog* dlg = new EchoRegistDialog();
   dlg->addHandler(session_timer_f->getHandler(dlg));
/* not work
   AmSessionEventHandlerFactory* efact = AmPlugIn::instance()->getFactory4Seh("echo");
   if(efact != NULL ) {
     AmSessionEventHandler* h = efact->getHandler(dlg);
     if (h != NULL )
        dlg->addHandler(h);
    }
*/
    
   return dlg;
}

void EchoRegistFactory::postEvent(AmEvent* ev) {
    dialer.postEvent(ev);
}

//
// EchoRegistDialog class
//

EchoRegistDialog::EchoRegistDialog()
{

}

EchoRegistDialog::~EchoRegistDialog()
{
}

void EchoRegistDialog::onSessionStart(const AmSipRequest& req)
{
    DBG("EchoRegistDialog::onSessionStart\n");
    setInOut(&echo,&echo);
//    startSession();
}

void EchoRegistDialog::onSessionStart(const AmSipReply& rep)
{
    DBG("EchoRegistDialog::onSessionStart (SEMS originator mode)\n");
    startSession();
}

void EchoRegistDialog::startSession(){
    // disable DTMF detection - don't use DTMF here
    setDtmfDetectionEnabled(false);

}

void EchoRegistDialog::onBye(const AmSipRequest& req)
{
    DBG("onBye: stopSession\n");
    setStopped();
}

void EchoRegistDialog::process(AmEvent* event)
{
    AmSession::process(event);
}

//
// RegThread class
//

void RegThread::add_reg(const RegInfo& ri) {
  registrations.push_back(ri);
}

void RegThread::create_registration(RegInfo& ri) {
  AmDynInvokeFactory* di_f = AmPlugIn::instance()->getFactory4Di("registrar_client");
  if (di_f == NULL) {
    ERROR("unable to get a registrar_client\n");
  } else {
    AmDynInvoke* uac_auth_i = di_f->getInstance();
    if (uac_auth_i!=NULL) {

      DBG("calling createRegistration\n");
      AmArg di_args, reg_handle;
      di_args.push(ri.domain.c_str());
      di_args.push(ri.user.c_str());
      di_args.push(ri.display_name.c_str()); // display name
      di_args.push(ri.auth_user.c_str());  // auth_user
      di_args.push(ri.passwd.c_str());    // pwd
      di_args.push(MOD_NAME); //sess_link
                        
      uac_auth_i->invoke("createRegistration", di_args, reg_handle);
      if (reg_handle.size()) 
        ri.handle = reg_handle.get(0).asCStr();
    }
  }
}

bool RegThread::check_registration(const RegInfo& ri) {
  if (!ri.handle.length())
    return false;
  AmDynInvokeFactory* di_f = AmPlugIn::instance()->getFactory4Di("registrar_client");
  if (di_f == NULL) {
    ERROR("unable to get a registrar_client\n");
  } else {
    AmDynInvoke* uac_auth_i = di_f->getInstance();
    if (uac_auth_i!=NULL) {

      AmArg di_args, res;
      di_args.push(ri.handle.c_str());
      uac_auth_i->invoke("getRegistrationState", di_args, res);
      if (res.size()) {
        if (!res.get(0).asInt())
          return false; // does not exist
        int state = res.get(1).asInt();
        int expires = res.get(2).asInt();
        DBG("Got state %s with expires %us for registration.\n", 
            getSIPRegistationStateString(state), expires);
        if (state == 2) // expired ... FIXME: add values from API here
          return false;
        // else pending or active
        return true;
      }
    }
  }
  return false;
}

void RegThread::run() {
  DBG("registrar client started.\n");
  sleep(2); // wait for sems to completely start up

  while (true) {
    for (vector<RegInfo>::iterator it = registrations.begin(); 
         it != registrations.end(); it++) {
      if (!check_registration(*it)) {
        DBG("Registration %d does not exist or timeout. Creating registration.\n",
            (int)(it - registrations.begin()));
        create_registration(*it);
      }
    }
    sleep(10); // 10 seconds
  }
                
}

void RegThread::on_stop() {
  DBG("not stopping...\n");
}

void RegThread::postEvent(AmEvent* ev) {
  DBG("received registration event.\n"); 
  // TODO: handle events instead of re-try
  delete ev;
}
